// David Pilling August 2015
// Most of code from 
// http://forum.arduino.cc/index.php?topic=155589.msg1166639#msg1166639


#include <stdarg.h>

void pr(char *fmt, ... )
{

  
  char buf[128]; // resulting string limited to 128 chars
  va_list args;
  va_start (args, fmt );
  vsnprintf(buf, 128, fmt, args);
  va_end (args);
  Serial.println(buf);
}





void setup_serial(void)
{
 Serial.begin(9600);

}




// Inputs
// AIN0 (digital pin 6) - potentiometer
// AIN1 (digital pin 7) - clock output

 
#define AIN0 6
#define AIN1 7





volatile unsigned t1captured = 0;
volatile unsigned t1capval = 0;
volatile unsigned t1ovfcnt = 0;
volatile unsigned long t1last = 0;

#define BUFFER_SIZE 32
#define BUFFER_MASK 31

volatile unsigned long int buffer[BUFFER_SIZE];
volatile byte head = 0;
volatile byte tail = 0;

void capsetup() 
{

 pinMode(AIN0, INPUT);
 pinMode(AIN1, INPUT);
 DIDR1  |=  (1<< AIN0D);         // disable digital input buffer AIN0/1
 DIDR1  |=  (1<< AIN1D);

 ADCSRA=0; // ADC Control and Status Register A

 TCCR1A = 0x0;    // put timer1 in normal mode
 TCCR1B = 0x2;    // change prescaler to divide clock by 8

 // clear any pending capture or overflow interrupts
 TIFR1 = (1<<ICF1) | (1<<TOV1);
 // Enable input capture and overflow interrupts
 TIMSK1 |= (1<<ICIE1) | (1<<TOIE1);

 // Analog Comparator Input Capture by Timer/Counter 1: On
 ADCSRB=0; // turn off muliplxer
 ACSR=(1<<ACIC); // Analog Comparator as capture trigger
}

void caploop() 
{

 byte newhead;

 if(head != tail) {
   newhead = (head + 1) & BUFFER_MASK;
   unsigned int seconds=millis()/1000;
   seconds=seconds % 10;
   long error=(buffer[newhead]-buffer[(newhead-1) & BUFFER_MASK])/2;
   
   pr("%ld",error);
   head = newhead;
 }
}

ISR(TIMER1_OVF_vect) { 
  t1ovfcnt++;              // keep track of overflows
}

ISR(TIMER1_CAPT_vect) {
 
 unsigned long t1temp;

 // combine overflow count with capture value to create 32 bit count
 //  calculate how long it has been since the last capture
 //   stick the result in the global variable t1time in 1uS precision
 t1capval = ICR1;
 t1temp = ((unsigned long)t1ovfcnt << 16) | t1capval;

 TCCR1B^=(1<<ICES1);

 if((t1temp-t1last)>0)
 {
  t1last = t1temp;
  tail = (tail + 1) & BUFFER_MASK;
  buffer[tail] = t1last;
 } 
}



void setup () 
{
 setup_serial();

 capsetup();

}



void loop () 
{

  caploop();
}
